@@ -1,6 +1,8 @@
 import logging
 import os
 
+from asgiref.sync import sync_to_async
+
 from aiohttp import web
 from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
@@ -11,7 +13,7 @@ from pulpcore.plugin.models import ContentArtifact
 from pulp_container.app.models import ContainerDistribution, Tag
 from pulp_container.app.schema_convert import Schema2toSchema1ConverterWrapper
 from pulp_container.app.utils import get_accepted_media_types
-from pulp_container.constants import MEDIA_TYPE
+from pulp_container.constants import EMPTY_BLOB, MEDIA_TYPE
 
 log = logging.getLogger(__name__)
 
@@ -19,14 +21,6 @@ v2_headers = MultiDict()
 v2_headers["Docker-Distribution-API-Version"] = "registry/2.0"
 
 
-class ArtifactNotFound(Exception):
-    """
-    The artifact associated with a published-artifact does not exist.
-    """
-
-    pass
-
-
 class Registry(Handler):
     """
     A set of handlers for the Container Registry v2 API.
@@ -97,15 +91,19 @@ class Registry(Handler):
                 streamed back to the client.
 
         """
+        self._reset_db_connection()
+
         path = request.match_info["path"]
         tag_name = request.match_info["tag_name"]
-        distribution = self._match_distribution(path)
-        self._permit(request, distribution)
-        repository_version = distribution.get_repository_version()
+        distribution = await sync_to_async(self._match_distribution)(path)
+        await sync_to_async(self._permit)(request, distribution)
+        repository_version = await sync_to_async(distribution.get_repository_version)()
         accepted_media_types = get_accepted_media_types(request.headers)
 
         try:
-            tag = Tag.objects.get(pk__in=repository_version.content, name=tag_name)
+            tag = await sync_to_async(Tag.objects.select_related("tagged_manifest").get)(
+                pk__in=await sync_to_async(repository_version.get_content)(), name=tag_name
+            )
         except ObjectDoesNotExist:
             raise PathNotResolved(tag_name)
 
@@ -129,7 +127,7 @@ class Registry(Handler):
                 "Content-Type": return_media_type,
                 "Docker-Content-Digest": tag.tagged_manifest.digest,
             }
-            return await Registry.dispatch_tag(tag, response_headers)
+            return await self.dispatch_tag(request, tag, response_headers)
 
         # return what was found in case media_type is accepted header (docker, oci)
         if tag.tagged_manifest.media_type in accepted_media_types:
@@ -138,17 +136,18 @@ class Registry(Handler):
                 "Content-Type": return_media_type,
                 "Docker-Content-Digest": tag.tagged_manifest.digest,
             }
-            return await Registry.dispatch_tag(tag, response_headers)
+            return await self.dispatch_tag(request, tag, response_headers)
 
         # convert if necessary
         return await Registry.dispatch_converted_schema(tag, accepted_media_types, path)
 
-    @staticmethod
-    async def dispatch_tag(tag, response_headers):
+    async def dispatch_tag(self, request, tag, response_headers):
         """
-        Finds an artifact associated with a Tag and sends it to the client.
+        Finds an artifact associated with a Tag and sends it to the client, otherwise tries
+        to stream it.
 
         Args:
+            request(:class:`~aiohttp.web.Request`): The request to prepare a response for.
             tag: Tag
             response_headers (dict): dictionary that contains the 'Content-Type' header to send
                 with the response
@@ -159,9 +158,10 @@ class Registry(Handler):
 
         """
         try:
-            artifact = tag.tagged_manifest._artifacts.get()
+            artifact = await sync_to_async(tag.tagged_manifest._artifacts.get)()
         except ObjectDoesNotExist:
-            raise ArtifactNotFound(tag.name)
+            ca = await sync_to_async(lambda x: x[0])(tag.tagged_manifest.contentartifact_set.all())
+            return await self._stream_content_artifact(request, web.StreamResponse(), ca)
         else:
             return await Registry._dispatch(artifact.file, response_headers)
 
@@ -188,7 +188,7 @@ class Registry(Handler):
         """
         schema1_converter = Schema2toSchema1ConverterWrapper(tag, accepted_media_types, path)
         try:
-            result = schema1_converter.convert()
+            result = await sync_to_async(schema1_converter.convert)()
         except RuntimeError:
             raise PathNotResolved(tag.name)
 
@@ -203,19 +203,26 @@ class Registry(Handler):
         """
         Return a response to the "GET" action.
         """
+        self._reset_db_connection()
 
         path = request.match_info["path"]
         digest = "sha256:{digest}".format(digest=request.match_info["digest"])
-        distribution = self._match_distribution(path)
-        self._permit(request, distribution)
-        repository_version = distribution.get_repository_version()
+        distribution = await sync_to_async(self._match_distribution)(path)
+        await sync_to_async(self._permit)(request, distribution)
+        repository_version = await sync_to_async(distribution.get_repository_version)()
+        if digest == EMPTY_BLOB:
+            return await Registry._empty_blob()
         try:
-            ca = ContentArtifact.objects.get(
-                content__in=repository_version.content, relative_path=digest
+            ca = await sync_to_async(
+                ContentArtifact.objects.select_related("artifact", "content").get
+            )(
+                content__in=await sync_to_async(repository_version.get_content)(),
+                relative_path=digest,
             )
+            ca_content = await sync_to_async(ca.content.cast)()
             headers = {
-                "Content-Type": ca.content.cast().media_type,
-                "Docker-Content-Digest": ca.content.cast().digest,
+                "Content-Type": ca_content.media_type,
+                "Docker-Content-Digest": ca_content.digest,
             }
         except ObjectDoesNotExist:
             raise PathNotResolved(path)
@@ -225,3 +232,19 @@ class Registry(Handler):
                 return await Registry._dispatch(artifact.file, headers)
             else:
                 return await self._stream_content_artifact(request, web.StreamResponse(), ca)
+
+    @staticmethod
+    async def _empty_blob():
+        # fmt: off
+        empty_tar = [
+            31, 139, 8, 0, 0, 9, 110, 136, 0, 255, 98, 24, 5, 163, 96, 20, 140, 88, 0, 8, 0, 0, 255,
+            255, 46, 175, 181, 239, 0, 4, 0, 0,
+        ]
+        # fmt: on
+        body = bytearray(empty_tar)
+        response_headers = {
+            "Docker-Content-Digest": EMPTY_BLOB,
+            "Content-Type": MEDIA_TYPE.REGULAR_BLOB,
+            "Docker-Distribution-API-Version": "registry/2.0",
+        }
+        return web.Response(body=body, headers=response_headers)
