@@ -25,7 +25,7 @@ from guardian.shortcuts import get_objects_for_user
 
 from pulpcore.plugin.models import Artifact, ContentArtifact, Task, UploadChunk
 from pulpcore.plugin.files import PulpTemporaryUploadedFile
-from pulpcore.plugin.tasking import add_and_remove, enqueue_with_reservation
+from pulpcore.plugin.tasking import add_and_remove, dispatch
 from rest_framework.exceptions import (
     AuthenticationFailed,
     NotAuthenticated,
@@ -48,13 +48,18 @@ from rest_framework.views import APIView
 from pulp_container.app import models, serializers
 from pulp_container.app.access_policy import RegistryAccessPolicy
 from pulp_container.app.authorization import AuthorizationService
-from pulp_container.app.redirects import FileStorageRedirects, S3StorageRedirects
+from pulp_container.app.redirects import (
+    FileStorageRedirects,
+    S3StorageRedirects,
+    AzureStorageRedirects,
+)
 from pulp_container.app.token_verification import (
     RegistryAuthentication,
     TokenAuthentication,
     RegistryPermission,
     TokenPermission,
 )
+from pulp_container.constants import EMPTY_BLOB
 
 FakeView = namedtuple("FakeView", ["action", "get_object"])
 
@@ -148,13 +153,12 @@ class UploadResponse(Response):
     This response object provides information about Uploads during 'push' operations.
     """
 
-    def __init__(self, upload, path, content_length, request):
+    def __init__(self, upload, path, request):
         """
         Args:
             upload (pulp_container.app.models.Upload): An Upload model used to generate the
                 response.
             path (str): The base_path of the ContainerDistribution (Container repository name)
-            content_length (int): The value for the Content-Length header.
             request (rest_framework.request.Request): Request object not used by this
                 implementation of Response.
         """
@@ -162,7 +166,7 @@ class UploadResponse(Response):
             "Docker-Upload-UUID": upload.pk,
             "Location": f"/v2/{path}/blobs/uploads/{upload.pk}",
             "Range": "0-{offset}".format(offset=int(upload.size - 1)),
-            "Content-Length": content_length,
+            "Content-Length": 0,
         }
         super().__init__(headers=headers, status=202)
 
@@ -182,12 +186,10 @@ class ManifestResponse(Response):
                 implementation of Response.
             status (int): Status code to send with the response.
         """
-        artifact = manifest._artifacts.get()
-        size = artifact.size
         headers = {
             "Docker-Content-Digest": manifest.digest,
             "Location": "/v2/{path}/manifests/{digest}".format(path=path, digest=manifest.digest),
-            "Content-Length": size,
+            "Content-Length": 0,
         }
         super().__init__(headers=headers, status=status, content_type=manifest.media_type)
 
@@ -214,7 +216,7 @@ class BlobResponse(Response):
             "Location": "/v2/{path}/blobs/{digest}".format(path=path, digest=blob.digest),
             "Etag": blob.digest,
             "Range": "0-{offset}".format(offset=int(size)),
-            "Content-Length": size,
+            "Content-Length": 0,
             "Content-Type": "application/octet-stream",
             "Connection": "close",
         }
@@ -592,7 +594,7 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
 
         upload = models.Upload(repository=repository, size=0)
         upload.save()
-        response = UploadResponse(upload=upload, path=path, content_length=0, request=request)
+        response = UploadResponse(upload=upload, path=path, request=request)
 
         return response
 
@@ -625,7 +627,7 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
             upload.size += chunk.size
             upload.save()
 
-        return UploadResponse(upload=upload, path=path, content_length=chunk.size, request=request)
+        return UploadResponse(upload=upload, path=path, request=request)
 
     def put(self, request, path, pk=None):
         """
@@ -646,7 +648,7 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
             try:
                 task = Task.objects.filter(
                     name__endswith="add_and_remove",
-                    reserved_resources_record__resource=f"upload:{pk}",
+                    reserved_resources_record=[f"upload:{pk}"],
                 ).last()
             except Task.DoesNotExist:
                 # No upload and no task for it => the upload probably never existed
@@ -658,10 +660,11 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
                 blob = models.Blob.objects.get(digest=digest)
                 return BlobResponse(blob, path, 201, request)
             elif task.state in ["waiting", "running"]:
-                raise Throttled(wait=5)
+                raise Throttled()
             else:
+                error = task.error
                 task.delete()
-                raise Exception("Failed.")
+                raise Exception(str(error))
 
         chunks = UploadChunk.objects.filter(upload=upload).order_by("offset")
 
@@ -678,11 +681,13 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
                 artifact.save()
             except IntegrityError:
                 artifact = Artifact.objects.get(sha256=artifact.sha256)
+                artifact.touch()
             try:
                 blob = models.Blob(digest=digest, media_type=models.MEDIA_TYPE.REGULAR_BLOB)
                 blob.save()
             except IntegrityError:
                 blob = models.Blob.objects.get(digest=digest)
+                blob.touch()
             try:
                 blob_artifact = ContentArtifact(
                     artifact=artifact, content=blob, relative_path=digest
@@ -693,12 +698,13 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
 
             upload.delete()
 
-            job = enqueue_with_reservation(
+            dispatched_task = dispatch(
                 add_and_remove,
-                [f"upload:{pk}", repository],
+                shared_resources=[f"upload:{pk}"],
+                exclusive_resources=[repository],
                 kwargs={
-                    "repository_pk": repository.pk,
-                    "add_content_units": [blob.pk],
+                    "repository_pk": str(repository.pk),
+                    "add_content_units": [str(blob.pk)],
                     "remove_content_units": [],
                 },
             )
@@ -706,16 +712,17 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
             # Wait a small amount of time
             for dummy in range(3):
                 time.sleep(1)
-                task = Task.objects.get(pk=job.id)
+                task = Task.objects.get(pk=dispatched_task.pk)
                 if task.state == "completed":
                     task.delete()
                     return BlobResponse(blob, path, 201, request)
                 elif task.state in ["waiting", "running"]:
                     continue
                 else:
+                    error = task.error
                     task.delete()
-                    raise Exception("Failed.")
-            raise Throttled(wait=5)
+                    raise Exception(str(error))
+            raise Throttled()
         else:
             raise Exception("The digest did not match")
 
@@ -735,6 +742,8 @@ class RedirectsMixin:
             self.redirects_class = FileStorageRedirects
         elif settings.DEFAULT_FILE_STORAGE == "storages.backends.s3boto3.S3Boto3Storage":
             self.redirects_class = S3StorageRedirects
+        elif settings.DEFAULT_FILE_STORAGE == "storages.backends.azure_storage.AzureStorage":
+            self.redirects_class = AzureStorageRedirects
         else:
             raise NotImplementedError()
 
@@ -760,6 +769,8 @@ class Blobs(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
         try:
             blob = models.Blob.objects.get(digest=pk, pk__in=repository_version.content)
         except models.Blob.DoesNotExist:
+            if pk == EMPTY_BLOB:
+                return redirects.redirect_to_content_app("blobs", pk)
             raise BlobNotFound(digest=pk)
         return redirects.issue_blob_redirect(blob)
 
@@ -822,6 +833,7 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
             manifest.save()
         except IntegrityError:
             manifest = models.Manifest.objects.get(digest=manifest.digest)
+            manifest.touch()
         ca = ContentArtifact(artifact=artifact, content=manifest, relative_path=manifest.digest)
         try:
             ca.save()
@@ -841,33 +853,35 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
             tag.save()
         except IntegrityError:
             tag = models.Tag.objects.get(name=tag.name, tagged_manifest=manifest)
+            tag.touch()
 
         tags_to_remove = models.Tag.objects.filter(
             pk__in=repository.latest_version().content.all(), name=tag
         ).exclude(tagged_manifest=manifest)
-        job = enqueue_with_reservation(
+        dispatched_task = dispatch(
             add_and_remove,
-            [repository],
+            exclusive_resources=[repository],
             kwargs={
-                "repository_pk": repository.pk,
-                "add_content_units": [tag.pk, manifest.pk],
-                "remove_content_units": tags_to_remove.values_list("pk"),
+                "repository_pk": str(repository.pk),
+                "add_content_units": [str(tag.pk), str(manifest.pk)],
+                "remove_content_units": [str(pk) for pk in tags_to_remove.values_list("pk")],
             },
         )
 
         # Wait a small amount of time
         for dummy in range(3):
             time.sleep(1)
-            task = Task.objects.get(pk=job.id)
+            task = Task.objects.get(pk=dispatched_task.pk)
             if task.state == "completed":
                 task.delete()
                 return ManifestResponse(manifest, path, request, status=201)
             elif task.state in ["waiting", "running"]:
                 continue
             else:
+                error = task.error
                 task.delete()
-                raise Exception("Failed.")
-        raise Throttled(wait=5)
+                raise Exception(str(error))
+        raise Throttled()
 
     def receive_artifact(self, chunk):
         """Handles assembling of Manifest as it's being uploaded."""
@@ -893,4 +907,5 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
                 artifact.save()
             except IntegrityError:
                 artifact = Artifact.objects.get(sha256=artifact.sha256)
+                artifact.touch()
             return artifact
