@@ -300,7 +300,10 @@ def compute_digest(manifest_data):
     """
     Compute the digest from the passed manifest data.
     """
-    return hashlib.sha256(manifest_data.encode("utf-8")).hexdigest()
+
+    hexdigest = hashlib.sha256(manifest_data.encode("utf-8")).hexdigest()
+    digest = "sha256:{}".format(hexdigest)
+    return digest
 
 
 def _get_config_dict(manifest):
@@ -312,7 +315,10 @@ def _get_config_dict(manifest):
 
 
 def _get_manifest_dict(manifest):
-    manifest_artifact = manifest._artifacts.get()
+    try:
+        manifest_artifact = manifest._artifacts.get()
+    except ObjectDoesNotExist:
+        raise RuntimeError()
     return _get_dict(manifest_artifact)
 
 
