@@ -1,5 +1,6 @@
 from gettext import gettext as _
 import os
+import re
 
 from django.db import IntegrityError
 from django.core.exceptions import ObjectDoesNotExist
@@ -21,7 +22,8 @@ from pulpcore.plugin.serializers import (
     RelatedField,
     RemoteSerializer,
     RepositorySerializer,
-    RepositoryVersionDistributionSerializer,
+    DistributionSerializer,
+    RepositoryVersionRelatedField,
     SingleArtifactContentSerializer,
     validate_unknown_fields,
 )
@@ -29,6 +31,7 @@ from pulpcore.plugin.serializers import (
 from . import models
 
 VALID_TAG_REGEX = r"^[A-Za-z0-9][A-Za-z0-9._-]*$"
+VALID_BASE_PATH_REGEX_COMPILED = re.compile(r"^[a-z0-9]+(?:(?:(?:[._]|__|[-]*)[a-z0-9]+)+)?$")
 
 
 class _GetOrCreateMixin:
@@ -163,7 +166,7 @@ class ContainerPushRepositorySerializer(RepositorySerializer):
     """
 
     class Meta:
-        fields = RepositorySerializer.Meta.fields
+        fields = tuple(set(RepositorySerializer.Meta.fields) - set(["remote"]))
         model = models.ContainerPushRepository
 
 
@@ -216,7 +219,7 @@ class ContainerRemoteSerializer(RemoteSerializer):
         model = models.ContainerRemote
 
 
-class ContainerDistributionSerializer(RepositoryVersionDistributionSerializer):
+class ContainerDistributionSerializer(DistributionSerializer):
     """
     A serializer for ContainerDistribution.
     """
@@ -245,6 +248,9 @@ class ContainerDistributionSerializer(RepositoryVersionDistributionSerializer):
     description = serializers.CharField(
         help_text=_("An optional description."), required=False, allow_null=True
     )
+    repository_version = RepositoryVersionRelatedField(
+        required=False, help_text=_("RepositoryVersion to be served"), allow_null=True
+    )
 
     def validate(self, data):
         """
@@ -284,9 +290,24 @@ class ContainerDistributionSerializer(RepositoryVersionDistributionSerializer):
                 )
         return validated_data
 
+    def validate_base_path(self, value):
+        """Check whether the passed repository base path is valid or not."""
+        if len(value) > 255:
+            raise serializers.ValidationError(
+                _("The entered base path cannot be longer than 255 characters.")
+            )
+
+        if not all(re.match(VALID_BASE_PATH_REGEX_COMPILED, p) for p in value.split("/")):
+            raise serializers.ValidationError(
+                _("The provided base path contains forbidden characters.")
+            )
+
+        return value
+
     class Meta:
         model = models.ContainerDistribution
-        fields = tuple(set(RepositoryVersionDistributionSerializer.Meta.fields) - {"base_url"}) + (
+        fields = tuple(set(DistributionSerializer.Meta.fields) - {"base_url"}) + (
+            "repository_version",
             "registry_path",
             "namespace",
             "private",
@@ -367,6 +388,7 @@ class TagImageSerializer(TagOperationSerializer):
             manifest = models.Manifest.objects.get(
                 pk__in=new_data["latest_version"].content.all(), digest=new_data["digest"]
             )
+            manifest.touch()
         except models.Manifest.DoesNotExist:
             raise serializers.ValidationError(
                 _(
@@ -603,7 +625,9 @@ class OCIBuildImageSerializer(serializers.Serializer):
                     _("Only one of 'containerfile' and 'containerfile_artifact' may be specified.")
                 )
             data["containerfile_artifact"] = Artifact.init_and_validate(data.pop("containerfile"))
-        elif "containerfile_artifact" not in data:
+        elif "containerfile_artifact" in data:
+            data["containerfile_artifact"].touch()
+        else:
             raise serializers.ValidationError(
                 _("'containerfile' or 'containerfile_artifact' must " "be specified.")
             )
@@ -622,6 +646,7 @@ class OCIBuildImageSerializer(serializers.Serializer):
                 )
                 try:
                     artifact = artifactfield.run_validation(data=url)
+                    artifact.touch()
                     artifacts[artifact.pk] = relative_path
                 except serializers.ValidationError as e:
                     # Append the URL of missing Artifact to the error message
