@@ -2,8 +2,7 @@
 """Tests that CRUD container content units."""
 import unittest
 
-from pulp_smash.pulp3.bindings import monitor_task
-from pulp_smash.pulp3.utils import delete_orphans
+from pulp_smash.pulp3.bindings import delete_orphans, monitor_task
 
 from pulp_container.tests.functional.utils import (
     gen_artifact,
