@@ -5,9 +5,11 @@ import unittest
 from urllib.parse import urlparse
 
 from pulp_smash import cli, config, exceptions
-from pulp_smash.pulp3.bindings import monitor_task
-from pulp_smash.pulp3.utils import delete_orphans
-
+from pulp_smash.pulp3.bindings import (
+    delete_orphans,
+    monitor_task,
+    PulpTestCase,
+)
 from pulp_container.tests.functional.api import rbac_base
 from pulp_container.tests.functional.constants import DOCKERHUB_PULP_FIXTURE_1
 from pulp_container.tests.functional.utils import (
@@ -24,7 +26,7 @@ from pulpcore.client.pulp_container import (
 )
 
 
-class PushRepoTestCase(unittest.TestCase, rbac_base.BaseRegistryTest):
+class PushRepoTestCase(PulpTestCase, rbac_base.BaseRegistryTest):
     """Verify whether images can be pushed to pulp."""
 
     @classmethod
@@ -285,6 +287,24 @@ class PushRepoTestCase(unittest.TestCase, rbac_base.BaseRegistryTest):
         with self.assertRaises(exceptions.CalledProcessError):
             self._push(image_path, invalid_local_url, self.user_helpless)
 
+        # test you can create distribution under the namespace that matches login
+        repo_name2 = f"{namespace_name}/matching2"
+        distribution = {"name": repo_name2, "base_path": repo_name2, "private": True}
+        distribution_response = self.user_helpless["distribution_api"].create(distribution)
+        created_resources = monitor_task(distribution_response.task).created_resources
+        distribution = self.user_helpless["distribution_api"].read(created_resources[0])
+
+        # cleanup, namespace removal also removes related distributions
+        namespace = self.namespace_api.list(name=namespace_name).results[0]
+        namespace_response = self.namespace_api.delete(namespace.pulp_href)
+        monitor_task(namespace_response.task)
+
+        # test you can create distribution if namespace does not exist but matches login
+        distribution = {"name": repo_name, "base_path": repo_name, "private": True}
+        distribution_response = self.user_helpless["distribution_api"].create(distribution)
+        created_resources = monitor_task(distribution_response.task).created_resources
+        distribution = self.user_helpless["distribution_api"].read(created_resources[0])
+
         # cleanup, namespace removal also removes related distributions
         namespace = self.namespace_api.list(name=namespace_name).results[0]
         self.addCleanup(self.namespace_api.delete, namespace.pulp_href)
