@@ -7,8 +7,8 @@ import requests
 from requests.exceptions import HTTPError
 
 from pulp_smash import api, config
-from pulp_smash.pulp3.bindings import monitor_task
-from pulp_smash.pulp3.utils import delete_orphans, gen_distribution, gen_repo
+from pulp_smash.pulp3.bindings import delete_orphans, monitor_task
+from pulp_smash.pulp3.utils import gen_distribution, gen_repo
 
 from pulp_container.tests.functional.constants import DOCKERHUB_PULP_FIXTURE_1
 
